/*
	Library of the [Shop] Core v1.9.5
	Author: FrozDark
	www.hlmod.ru
*/

#if defined _shop_included
 #endinput
#endif
#define _shop_included

#define SHOP_VERSION	"1.9.5"
#define SHOP_MAX_STRING_LENGTH 64


enum ShopAction
{
	Shop_Raw = 0,
	Shop_Use = 1,
	Shop_ToggleOn = 2,
	Shop_ToggleOff = 3,
	Shop_NoItems = 4
}

enum ShopMenu
{
	Menu_Main,
	Menu_Buy,
	Menu_Sell,
	Menu_Inventory,
	Menu_Functions,
	Menu_AdditionalFunctions,
	Menu_AdminPanel,
	Menu_CreditsTransfer,
	Menu_ItemTransfer
}

enum ItemType
{
	Item_None = 0,		/**< Can not be used */
	Item_Finite = 1,	/**< Can be used like a medkit */
	Item_Togglable = 2	/**< Can be switched on or off */
}

functag public bool:CategoryDisplayCallback(client, const String:category[], const String:name[], String:buffer[], maxlen);
functag public bool:CategoryDisplayDescCallback(client, const String:category[], const String:description[], String:buffer[], maxlen);

functag public bool:ItemDisplayCallback(client, const String:category[], const String:item[], const String:name[], String:buffer[], maxlen);
functag public bool:ItemDisplayDescCallback(client, const String:category[], const String:item[], const String:description[], String:buffer[], maxlen);

functag public ItemElapseCallback(client, const String:category[], const String:item[], itemID);

functag public CategoryRegister(const String:category[], const String:name[], const String:description[]);

functag public bool:CategorySelect(client, const String:category[], ShopMenu:menu);

funcenum ItemUseToggleCallback
{
	ShopAction:public(client, const String:category[], const String:item[], itemID),
	ShopAction:public(client, const String:category[], const String:item[], itemID, bool:isOn)
}

functag public Action:ItemBuySellCallback(client, const String:category[], const String:item[], itemID, &credits);

functag public ItemRegister(const String:category[], const String:item[], itemID);

functag public CustomMenuDisplay(client, String:buffer[], maxlen);
functag public CustomMenuSelect(client);

native bool:Shop_RegisterCategory(const String:category[], const String:name[], const String:description[], 
								CategoryRegister:cat_register=CategoryRegister:INVALID_FUNCTION, 
								CategorySelect:cat_select=CategorySelect:INVALID_FUNCTION, 
								CategoryDisplayCallback:cat_name=CategoryDisplayCallback:INVALID_FUNCTION, 
								CategoryDisplayDescCallback:cat_desc=CategoryDisplayDescCallback:INVALID_FUNCTION, 
								team = 0);

/**
 *	Removes item from category
 *	-
 *	@param category			The category id
 *	@param item				The item name to remove
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_RemoveItemFromCategory(const String:category[], const String:item[]);

/**
 *	Start an item to register
 *	-
 *	@param category			The category id
 *	@param item				The item id to register
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_StartItem(const String:category[], const String:item[]);

/**
 *	Sets item info
 *	-
 *	@param name			Display name to set
 *	@param description			Item description
 *	@param price			Item price
 *	@param sell_price		Item sell price. -1 to make unsaleable
 *	@param type				Item type. See ItemType enumeration
 *	@param duration			Item duration
 *	@param team				Team index who can use item
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_SetItemInfo(const String:name[], const String:description[], price, sell_price = -1, ItemType:type = Item_None, duration = 0, team = 0);

/**
 *	Sets item custom info
 *	-
 *	@param info			Name of the info
 *	@param value			Value to set
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_SetItemCustomInfo(const String:info[], value);

/**
 *	Sets item custom info
 *	-
 *	@param info			Name of the info
 *	@param value			Value to set
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_SetItemCustomInfoFloat(const String:info[], Float:value);

/**
 *	Sets item custom info
 *	-
 *	@param info			Name of the info
 *	@param value			Value to set
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_SetItemCustomInfoString(const String:info[], String:value[]);

/**
 *	Sets item callbacks
 *	-
 *	@param use_toggle		Callback that will be called on uses/toggles item
 *	@param register			Callback that will be called on fully registering the item
 *	@param display			Callback that will be called on item being diplayed
 *	@param description		Callback that will be called on item description being diplayed
 *	@param buy				Callback that will be called when a client is trying to buy the item
 *	@param sell				Callback that will be called when a client is trying to sell the item
 *	@param elapse			Callback that will be called when the item item has been elapsed
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_SetItemCallbacks(ItemUseToggleCallback:use_toggle=ItemUseToggleCallback:INVALID_FUNCTION, 
								ItemRegister:register=ItemRegister:INVALID_FUNCTION, 
								ItemDisplayCallback:display=ItemDisplayCallback:INVALID_FUNCTION, 
								ItemDisplayDescCallback:description=ItemDisplayDescCallback:INVALID_FUNCTION, 
								ItemBuySellCallback:buy=ItemBuySellCallback:INVALID_FUNCTION, 
								ItemBuySellCallback:sell=ItemBuySellCallback:INVALID_FUNCTION,
								ItemElapseCallback:elapse=ItemElapseCallback:INVALID_FUNCTION);

/**
 *	Ends registering started item
 *	-
 *	@noparams
 *	-
 *	@return	True on success. False otherwise
*/
native bool:Shop_EndItem();

/**
 *	Gets the item custom info
 *	-
 *	@param category			The category id where item placed
 *	@param item				The item id
 *	@param info				Info to get
 *	@param defvalue			Optional default value if the info is not set
 *	-
 *	@return	Value on success or -1 if the item is not exists
*/
native Shop_GetItemCustomInfo(const String:category[], const String:item[], String:info[], defvalue = 0);

/**
 *	Gets the item custom info
 *	-
 *	@param category			The category id where item placed
 *	@param item				The item id
 *	@param info				Info to get
 *	@param defvalue			Optional default value if the info is not set
 *	-
 *	@return	Value on success or -1.0 if the item is not exists
*/
native Float:Shop_GetItemCustomInfoFloat(const String:category[], const String:item[], String:info[], Float:defvalue = 0.0);

/**
 *	Gets the item custom info
 *	-
 *	@param category			The category id where item placed
 *	@param item				The item id
 *	@param info				Info to get
 *	@param buffer			Buffer to store info in
 *	@param buffer_size		Max buffer length
 *	@param defvalue			Optional default value if the info is not set
 *	-
 *	@return	Number of bytes written
*/
native Shop_GetItemCustomInfoString(const String:category[], const String:item[], const String:info[], String:buffer[], buffer_size, const String:defvalue[] = "");

/**
 *	Adds an item to the functions menu
 *	-
 *	@param clbk_display		Callback that will be called on item being diplayed
 *	@param clbk_select		Callback that will be called on item selection
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_AddToFunctionsMenu(CustomMenuDisplay:clbk_display, CustomMenuSelect:clbk_select);

/**
 *	Removes an item from the functions menu
 *	-
 *	@param clbk		Function to remove
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_RemoveFromFunctionsMenu(Function:clbk);

/**
 *	Adds an item to the additional functions menu
 *	-
 *	@param clbk_display		Callback that will be called on item being diplayed
 *	@param clbk_select		Callback that will be called on item selection
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_AddToExtraFunctionsMenu(CustomMenuDisplay:clbk_display, CustomMenuSelect:clbk_select);

/**
 *	Removes an item from the additional functions menu
 *	-
 *	@param clbk		Function to remove
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_RemoveFromExtraFunctionsMenu(Function:clbk);

/**
 *	Adds an item to the admin menu
 *	-
 *	@param clbk_display		Callback that will be called on item being diplayed
 *	@param clbk_select		Callback that will be called on item selection
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_AddToAdminMenu(CustomMenuDisplay:clbk_display, CustomMenuSelect:clbk_select);

/**
 *	Removes an item from the admin menu
 *	-
 *	@param clbk		Function to remove
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_RemoveFromAdminMenu(Function:clbk);

/**
 *	Shows main menu
 *	-
 *	@param client		Client index to show to
 *	-
 *	@noreturn
*/
native Shop_ShowMainMenu(client);

/**
 *	Shows inventory menu
 *	-
 *	@param client		Client index to show to
 *	-
 *	@noreturn
*/
native Shop_ShowInventory(client);

/**
 *	Shows additional functions menu
 *	-
 *	@param client		Client index to show to
 *	-
 *	@noreturn
*/
native Shop_ShowFunctions(client);

/**
 *	Shows main menu
 *	-
 *	@param client		Client index to show to
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_ShowAdminMenu(client);

/**
 *	Called when the plugin connected to the database and ready to register categories/items
 *	-
 *	@noparams
 *	-
 *	@noreturn
*/
forward Shop_OnConnected();

/**
 *	Called when the plugin has loaded the player's data
 *	-
 *	@param client		Client index
 *	-
 *	@noreturn
 */
forward Shop_OnAuthorized(client);

/**
 *	Called when the plugin has loaded item to a client's inventory when he has connected
 *	-
 *	@param client		Client index
 *	@param category		The item category
 *	@param item			The item
 *	@param itemId		The item id
 *	-
 *	@noreturn
 */
forward Shop_OnItemLoaded(client, const String:category[], const String:item[], itemId);

/**
 *	Called when an item has started to register
 *	-
 *	@param category		Item category
 *	@param item			Item
 *	-
 *	@noreturn
 */
forward Shop_OnItemStarted(const String:category[], const String:item[]);

/**
 *	Called when a client is about to luck an item.
 *	-
 *	@param client		Client index
 *	@param category		The item category
 *	@param item			The item
 *	@param itemId		The item id
 *	-
 *	@return		Plugin_Continue to allow luck and any higher value to block. It's also doesn't take credits on block
 */
forward Action:Shop_OnItemLuck(client, const String:category[], const String:item[], itemId);

/**
 *	Called when a client has lucked an item.
 *	-
 *	@param client		Client index
 *	@param category		The item category
 *	@param item			The item
 *	@param itemId		The item id
 *	-
 *	@noreturn
 */
forward Shop_OnItemLucked(client, const String:category[], const String:item[], itemId);

/**
 *	Called when a menu is about to be titled
 *	-
 *	@param client		Client index
 *	@param menu			Menu type
 *	@param title		Title that is going to be
 *	@param buffer		Buffer to store new title in
 *	@param maxlength	Max buffer length
 *	-
 *	@return		true to aplly new parameters and false to ignore
 */
forward bool:Shop_OnMenuTitle(client, ShopMenu:menu, const String:title[], String:buffer[], maxlength);

/**
 *	Called when an item is being displayed
 *	-
 *	@param client		Client index to whom an item is being displayed
 *	@param menu			Menu type
 *	@param category		The item category
 *	@param item			The item
 *	@param itemId		The item id
 *	@param title		Display text that is going to be
 *	@param buffer		Buffer to store new text in
 *	@param maxlength	Max buffer length
 *	@param disable		true to make item disabled. By reference
 *	-
 *	@return		Plugin_Changed to apply new parameters, Plugin_Continue to ignore, any other to block item to be displayed
 */
forward Action:Shop_OnItemDisplay(client, ShopMenu:menu, const String:category[], const String:item[], itemId, const String:display[], String:buffer[], maxlength, &bool:disable);

/**
 *	Called when an item is being transefer
 *	-
 *	@param client		Client index who is trying to transfer
 *	@param target		Client index who is receiving an item
 *	@param cost			The transfer cost. Can be changed
 *	@param category		The item category
 *	@param item			The item
 *	@param itemId		The item id
 *	-
 *	@return		Plugin_Changed to apply new parameters, Plugin_Continue to ignore, any other to block item to be transfered
 */
forward Action:Shop_OnItemTransfer(client, target, &cost, const String:category[], const String:item[], itemId);

/**
 *	Checks whether the plugin connected to the database
 *	-
 *	@noparams
 *	-
 *	@return	True if the plugin connected to the database, false otherwise
*/
native bool:Shop_IsConnected();

/**
 *	Checks whether the client is the admin of shop
 *	-
 *	@noparams
 *	-
 *	@return	True if client is admin, false otherwise
*/
native bool:Shop_IsAdmin(client);

/**
 *	Checks whether the player has been loaded from database
 *	-
 *	@param client	Client index to check
 *	-
	@return	True if the player loaded, false otherwise
*/
native bool:Shop_IsAuthorized(client);

/**
 *	Manually inserts a player if auto insertion is disabled
 *	-
 *	@param client	Client index to insert
 *	-
 *	@return	True on success, false otherwise
*/
native bool:Shop_InsertPlayer(client);

/**
 *	Gets the client id
 *	-
 *	@param client			Client index to get from
 *	-
 *	@return	Client id
*/
native Shop_GetClientId(client);

/**
 *	Gets the database handle. Must be freed with CloseHandle()
 *	-
 *	@noparams
 *	-
 *	@return	Database Handle
*/
native Handle:Shop_GetDatabase();

/**
 *	Gets "Database prefix" parameter.
 *	-
 *	@param buffer			Buffer to store the prefix in
 *	@param maxlength		Max buffer length
 *	-
 *	@return	Number of bytes written
*/
native Shop_GetDatabasePrefix(String:buffer[], maxlength);

/**
 *	Gets the item price
 *	-
 *	@param category			The category id
 *	@param item				The item name to get from
 *	-
 *	@return	The item price or -1 if the item is not exists
*/
native Shop_GetItemPrice(const String:category[], const String:item[]);

/**
 *	Sets the item price
 *	-
 *	@param category			The category id
 *	@param item				The item name
 *	@param price			Price to set
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_SetItemPrice(const String:category[], const String:item[], price);

/**
 *	Gets the item selling price
 *	-
 *	@param category			The category id
 *	@param item				The item name to get from
 *	-
 *	@return	The item selling price or -1 if the item is not exists
*/
native Shop_GetItemSellPrice(const String:category[], const String:item[]);

/**
 *	Sets the item selling price
 *	-
 *	@param category			The category id
 *	@param item				The item name
 *	@param price			Price to set. -1 to use buying price or -2 to make unsaleable
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_SetItemSellPrice(const String:category[], const String:item[], price);

/**
 *	Gets the item duration
 *	-
 *	@param category			The category id
 *	@param item				The item name to get from
 *	-
 *	@return	The item duration or -1 if the item is not exists
*/
native Shop_GetItemDuration(const String:category[], const String:item[]);

/**
 *	Sets the item duration
 *	-
 *	@param category			The category id
 *	@param item				The item name
 *	@param duration			Duration to set. 0 to make the item forever
 *	-
 *	@return	true on success, false otherwise
*/
native bool:Shop_SetItemDuration(const String:category[], const String:item[], duration);

/**
 *	Gets the item id
 *	-
 *	@param category			The category id
 *	@param item				The item name to get from
 *	-
 *	@return	The item id or -1 on failure
*/
native Shop_GetItemId(const String:category[], const String:item[]);

/**
 *	Gets the item by id
 *	-
 *	@param category			The category id
 *	@param itemID			The item id to get the item for
 *	@param buffer			Buffer to store the item name
 *	@param size				Maxlength of buffer
 *	-
 *	@return	Number of bytes written
*/
native Shop_GetItemById(const String:category[], itemID, String:buffer[], size);

/**
 *	This must be called on PluginEnd. Fully unregisters the plugin. Unregisters items, categories and removes them from the shop and players' inventory
 *	-
 *	@noparams
 *	-
 *	@noreturn
*/
native Shop_UnregisterMe();

/**
 *	Give item to the player
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item to give
 *	-
 *	@return	True if the player has been successfuly given, false otherwise
*/
native bool:Shop_GiveItem(client, const String:category[], const String:item[]);

/**
 *	Give item to the player by item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item id to give
 *	-
 *	@return	True if the player has been successfuly given, false otherwise
*/
native bool:Shop_GiveItemId(client, const String:category[], itemID);

/**
 *	Forces the player to buy the item from the shop
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item to buy
 *	-
 *	@return	True if the player successfuly bought, false otherwise
*/
native bool:Shop_BuyItem(client, const String:category[], const String:item[]);

/**
 *	Forces the player to buy the item from the shop by the item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item id to buy
 *	-
 *	@return	True if the player successfuly bought, false otherwise
*/
native bool:Shop_BuyItemId(client, const String:category[], itemID);

/**
 *	Forces the player to sell the item from the inventory
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item to sell
 *	-
 *	@return	True if the player successfuly sold, false otherwise
*/
native bool:Shop_SellItem(client, const String:category[], const String:item[]);

/**
 *	Forces the player to sell the item from the inventory by the item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item id to buy
 *	-
 *	@return	True if the player successfuly sold, false otherwise
*/
native bool:Shop_SellItemId(client, const String:category[], itemID);

/**
 *	Forces the player to use the item from his inventory
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item to use
 *	-
 *	@return	True if the player has the item and successfuly used, false otherwise
*/
native bool:Shop_UseItem(client, const String:category[], const String:item[]);

/**
 *	Forces the player to use the item from his inventory by the item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item id to use
 *	-
 *	@return	True if the player has the item and successfuly used, false otherwise
*/
native bool:Shop_UseItemId(client, const String:category[], itemID);

/**
 *	Remove the item from the player's inventory
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item to remove
 *	-
 *	@return	True on success, false otherwise
*/
native bool:Shop_RemoveItem(client, const String:category[], const String:item[]);

/**
 *	Remove the item from the player's inventory by the item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item id to remove
 *	-
 *	@return	True on success, false otherwise
*/
native bool:Shop_RemoveItemId(client, const String:category[], itemID);

/**
 *	Checks whether the player has the item in his inventory
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item to check
 *	-
 *	@return	True if the player has the item, false otherwise
*/
native bool:Shop_HasItem(client, const String:category[], const String:item[]);

/**
 *	Checks whether the player has the item in his inventory by the item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item id to check
 *	-
 *	@return	True if the player has the item, false otherwise
*/
native bool:Shop_HasItemId(client, const String:category[], itemID);

/**
 *	Toggles the item of the player
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item to toggle
 *	@param value			True to toggle on and false to toggle off
 *	-
 *	@noreturn
*/
native Shop_ToggleItem(client, const String:category[], const String:item[], bool:value);

/**
 *	Toggles the item of the player by the item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item id to toggle
 *	@param value			True to toggle on and false to toggle off
 *	-
 *	@noreturn
*/
native Shop_ToggleItemId(client, const String:category[], itemID, bool:value);

/**
 *	Toggles all items of category off
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	-
 *	@noreturn
*/
native Shop_ToggleCategoryOff(client, const String:category[]);

/**
 *	Checks whether the player has the item toggled on in his inventory
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param item				Item to check
 *	-
 *	@return	True if the player has the item toggled on, false otherwise
*/
native bool:Shop_IsToggled(client, const String:category[], const String:item[]);

/**
 *	Checks whether the player has the item toggled on in his inventory by the item id
 *	-
 *	@param client			Client index
 *	@param category			The category id
 *	@param itemID			Item id to check
 *	-
 *	@return	True if the player has the item toggled on, false otherwise
*/
native bool:Shop_IsToggledId(client, const String:category[], itemID);

/**
 *	Gives certain amount of credits to the player
 *	-
 *	@param client			Client index
 *	@param amount			Amount to give
 *	-
 *	@return	New amount of credits, -1 on failure
*/
native Shop_GiveCredits(client, amount);

/**
 *	Takes certain amount of credits from the player
 *	-
 *	@param client			Client index
 *	@param amount			Amount to take
 *	-
 *	@return	New amount of credits, -1 on failure
*/
native Shop_RemoveCredits(client, amount);

/**
 *	Gets the amount of credits of the player
 *	-
 *	@param client			Client index
 *	-
 *	@return	Amount of credits a player has, -1 on failure
*/
native Shop_GetCredits(client);

/**
 *	Sets the amount of credits to the player
 *	-
 *	@param client			Client index
 *	@param credits			Amount of credits to set
 *	-
 *	@noreturn
*/
native Shop_SetCredits(client, credits);

/**
 *	Gets path to the main config folder of the Shop
 *	-
 *	@param buffer			Buffer to store the path in
 *	@param size				Max buffer length
 *	@param file				File to retrieve the path
 *	-
 *	@noreturn
*/
stock Shop_GetCfgFile(String:buffer[], size, const String:file[])
{
	BuildPath(Path_SM, buffer, size, "configs/shop/%s", file);
}

public SharedPlugin:__pl_shop = 
{
	name = "shop",
	file = "shop.smx",
#if defined REQUIRE_PLUGIN
	required = 1,
#else
	required = 0,
#endif
};

#if !defined REQUIRE_PLUGIN
public __pl_shop_SetNTVOptional()
{
	MarkNativeAsOptional("Shop_RegisterCategory");
	
	MarkNativeAsOptional("Shop_StartItem");
	MarkNativeAsOptional("Shop_SetItemInfo");
	MarkNativeAsOptional("Shop_SetItemCustomInfo");
	MarkNativeAsOptional("Shop_SetItemCustomInfoFloat");
	MarkNativeAsOptional("Shop_SetItemCustomInfoString");
	MarkNativeAsOptional("Shop_SetItemCallbacks");
	MarkNativeAsOptional("Shop_EndItem");
	
	MarkNativeAsOptional("Shop_GetItemCustomInfo");
	MarkNativeAsOptional("Shop_GetItemCustomInfoFloat");
	MarkNativeAsOptional("Shop_GetItemCustomInfoString");
	
	MarkNativeAsOptional("Shop_RemoveItemFromCategory");
	
	MarkNativeAsOptional("Shop_AddToFunctionsMenu");
	MarkNativeAsOptional("Shop_RemoveFromFunctionsMenu");
	
	MarkNativeAsOptional("Shop_AddToAdminMenu");
	MarkNativeAsOptional("Shop_RemoveFromAdminMenu");
	
	MarkNativeAsOptional("Shop_ShowMainMenu");
	MarkNativeAsOptional("Shop_ShowInventory");
	MarkNativeAsOptional("Shop_ShowFunctions");
	MarkNativeAsOptional("Shop_ShowAdminMenu");
	
	MarkNativeAsOptional("Shop_IsConnected");
	
	MarkNativeAsOptional("Shop_IsAdmin");
	MarkNativeAsOptional("Shop_IsAuthorized");
	MarkNativeAsOptional("Shop_InsertPlayer");
	
	MarkNativeAsOptional("Shop_GetClientId");
	MarkNativeAsOptional("Shop_GetDatabase");
	MarkNativeAsOptional("Shop_GetDatabasePrefix");
	
	MarkNativeAsOptional("Shop_GetItemPrice");
	MarkNativeAsOptional("Shop_SetItemPrice");
	
	MarkNativeAsOptional("Shop_GetItemSellPrice");
	MarkNativeAsOptional("Shop_SetItemSellPrice");
	
	MarkNativeAsOptional("Shop_GetItemDuration");
	MarkNativeAsOptional("Shop_SetItemDuration");
	
	MarkNativeAsOptional("Shop_GetItemId");
	MarkNativeAsOptional("Shop_GetItemById");
	
	MarkNativeAsOptional("Shop_UnregisterMe");
	
	MarkNativeAsOptional("Shop_GiveItem");
	MarkNativeAsOptional("Shop_GiveItemId");
	
	MarkNativeAsOptional("Shop_BuyItem");
	MarkNativeAsOptional("Shop_BuyItemId");
	
	MarkNativeAsOptional("Shop_SellItem");
	MarkNativeAsOptional("Shop_SellItemId");
	
	MarkNativeAsOptional("Shop_UseItem");
	MarkNativeAsOptional("Shop_UseItemId");
	
	MarkNativeAsOptional("Shop_RemoveItem");
	MarkNativeAsOptional("Shop_RemoveItemId");
	
	MarkNativeAsOptional("Shop_HasItem");
	MarkNativeAsOptional("Shop_HasItemId");
	
	MarkNativeAsOptional("Shop_ToggleItem");
	MarkNativeAsOptional("Shop_ToggleItemId");
	
	MarkNativeAsOptional("Shop_ToggleCategoryOff");
	
	MarkNativeAsOptional("Shop_IsToggled");
	MarkNativeAsOptional("Shop_IsToggledId");
	
	MarkNativeAsOptional("Shop_GiveCredits");
	MarkNativeAsOptional("Shop_RemoveCredits");
	MarkNativeAsOptional("Shop_GetCredits");
	MarkNativeAsOptional("Shop_SetCredits");
}
#endif